﻿using System;
using System.Drawing;
using System.Web.UI;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.WebEdition;

namespace Serco.Web.Atlas.Public
{
    public partial class WebForm3 : System.Web.UI.Page
    {
        private static RectangleShape englishExtents = new RectangleShape(-645000, 7530000, 210000, 6430000);

        /// <summary>
        /// Handles the Load event of the Page control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                this.SetupMap(SignsMap);
                this.SetupMap(CctvMap);
            }
        }

        /// <summary>
        /// Set up the specified map control.
        /// </summary>
        /// <param name="map">The map.</param>
        private void SetupMap(Map map)
        {
            // Set viewport, etc on map
            map.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
            map.CurrentExtent = englishExtents;
            map.CurrentExtent = englishExtents;

            map.MapUnit = GeographyUnit.Meter;
            map.MapTools.MouseCoordinate.Enabled = true;
            map.MapTools.OverlaySwitcher.Enabled = true;

            // Setup OSM layer on map
            OpenStreetMapOverlay osmOverlay = new OpenStreetMapOverlay("Open Street Map");
            map.CustomOverlays.Add(osmOverlay);

            // Define marker overlay for placing marker pins on map
            SimpleMarkerOverlay markerOverlay = new SimpleMarkerOverlay("MarkerOverlay" + map.ClientID);
            map.CustomOverlays.Add(markerOverlay);
        }

        /// <summary>
        /// Converts a latitude and longitude to a spherical mercator point.
        /// </summary>
        /// <param name="longitude">The longitude.</param>
        /// <param name="latitude">The latitude.</param>
        /// <returns></returns>
        public static Point LongLatToSphericalMercator(double longitude, double latitude)
        {
            // Spherical Mercator for Google, VE, Yahoo etc
            // epsg:900913 R=6378137, x=longitude, y= R*ln(tan(pi/4 + latitude/2)
            double x = 6378137.0 * Math.PI / 180 * longitude;
            double y = 6378137.0 * Math.Log(Math.Tan(Math.PI / 180 * (45 + latitude / 2.0)));
            return new Point(Convert.ToInt32(x), Convert.ToInt32(y));
        }

        /// <summary>
        /// Handles the Click event of the Button1 control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="args">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void Button1_Click(object sender, EventArgs args)
        {
            AddMarkersToMap(SignsMap);
            ZoomToPoints(SignsMap);

            AddMarkersToMap(CctvMap);
            ZoomToPoints(CctvMap);
        }

        /// <summary>
        /// Adds markers to the specified map.
        /// </summary>
        /// <param name="map">The map.</param>
        private void AddMarkersToMap(Map map)
        {
            // Grab the marker overlay
            SimpleMarkerOverlay markerOverlay = (SimpleMarkerOverlay)map.CustomOverlays["MarkerOverlay" + map.ClientID];

            Point newPoint1 = LongLatToSphericalMercator(-0.49601668119430542, 51.561748504638672);
            markerOverlay.Markers.Add(new Marker(newPoint1.X, newPoint1.Y, new WebImage(21, 25, -10.5f, -25f)));

            Point newPoint2 = LongLatToSphericalMercator(-0.5061333179473877, 51.544498443603516);
            markerOverlay.Markers.Add(new Marker(newPoint2.X, newPoint2.Y, new WebImage(21, 25, -10.5f, -25f)));

            Point newPoint3 = LongLatToSphericalMercator(-0.52176666259765625, 51.559700012207031);
            markerOverlay.Markers.Add(new Marker(newPoint3.X, newPoint3.Y, new WebImage(21, 25, -10.5f, -25f)));

        }

        /// <summary>
        /// Finds the bounding rectangle for the points currently present on the specifed map.
        /// </summary>
        /// <param name="map">The map.</param>
        private void ZoomToPoints(Map map)
        {
            // Check the points in the marker layer
            SimpleMarkerOverlay markerOverlay = (SimpleMarkerOverlay)map.CustomOverlays["MarkerOverlay" + map.ClientID];
            RectangleShape extent = null;

            foreach (Marker marker in markerOverlay.Markers)
            {
                PointShape pointShape = (PointShape)marker.Position;

                if (extent == null)
                {
                    extent = pointShape.GetBoundingBox();
                }
                else
                {
                    extent.ExpandToInclude(pointShape);
                }
            }

            map.CurrentExtent = extent;
        }
    }
}
